@@ -8,7 +8,7 @@ Net::WebSocket::Parser - Parse WebSocket from a filehandle
 
 =head1 SYNOPSIS
 
-    my $iof = IO::Framed::Read->new($fh);
+    my $iof = IO::Framed->new($fh);
 
     my $parse = Net::WebSocket::Parser->new($iof);
 
@@ -29,6 +29,9 @@ A call to this method yields one of the following:
 
 =item * If a frame can be read, it will be returned.
 
+=item * If we hit an empty read (i.e., indicative of end-of-file),
+empty string is returned.
+
 =item * If only a partial frame is ready, undef is returned.
 
 =back
@@ -38,9 +41,21 @@ A call to this method yields one of the following:
 L<IO::Framed> was born out of work on this module; see that module’s
 documentation for the particulars of working with it. In particular,
 note the exceptions L<IO::Framed::X::EmptyRead> and
-L<IO::Framed::X::ReadError>. (As described in L<Net::WebSocket>’s
-documentation, you can use an equivalent interface for frame chunking if you
-wish.)
+L<IO::Framed::X::ReadError>.
+
+Again, you can use an equivalent interface for frame chunking if you wish.
+
+=head1 CONCERNING EMPTY READS
+
+An empty read is how we detect that a file handle (or socket, etc.) has no
+more data to read. Generally we shouldn’t get this in WebSocket since it
+means that a peer endpoint has gone away without sending a close frame.
+It is thus recommended that applications regard an empty read on a WebSocket
+stream as an error condition; e.g., if you’re using L<IO::Framed::Read>,
+you should NOT enable the C<allow_empty_read()> behavior.
+
+Nevertheless, this module (and L<Net::WebSocket::Endpoint>) do work when
+that flag is enabled.
 
 =head1 CUSTOM FRAMES SUPPORT
 
@@ -92,68 +107,65 @@ sub new {
 
     return bless {
         _reader => $reader,
+        _partial_frame => q<>,
     }, $class;
 }
 
+#Create these out here so that we don’t create/destroy them on each frame.
+#As long as we don’t access them prior to writing to them this is fine.
+my ($oct1, $oct2, $len, $mask_size, $len_len, $longs, $long);
+
 sub get_next_frame {
     my ($self) = @_;
 
     local $@;
 
-    if (!exists $self->{'_partial_frame'}) {
-        $self->{'_partial_frame'} = q<>;
-    }
-
     #It is really, really inconvenient that Perl has no “or” operator
     #that considers q<> falsey but '0' truthy. :-/
     #That aside, if indeed all we read is '0', then we know that’s not
     #enough, and we can return.
-    my $first2 = $self->_read_with_buffer(2) or return undef;
+    my $first2 = $self->_read_with_buffer(2);
+    if (!$first2) {
+        return defined($first2) ? q<> : undef;
+    }
 
-    #Now that we’ve read our header bytes, we’ll read some more.
-    #There may not actually be anything to read, though, in which case
-    #some readers will error (e.g., EAGAIN from a non-blocking filehandle).
-    #From a certain ideal we’d return #on each individual read to allow
-    #the reader to wait until there is more data ready; however, for
-    #practicality (and speed) let’s go ahead and try to read the rest of
-    #the frame. That means we need to set some flag to let the reader know
-    #not to die() if there’s no more data currently, as we’re probably
-    #expecting more soon to complete the frame.
-    local $self->{'_reading_frame'} = 1;
+    ($oct1, $oct2) = unpack('CC', $first2 );
 
-    my ($oct1, $oct2) = unpack('CC', $first2 );
+    $len = $oct2 & 0x7f;
 
-    my $len = $oct2 & 0x7f;
+    $mask_size = ($oct2 & 0x80) && 4;
 
-    my $mask_size = ($oct2 & 0x80) && 4;
+    $len_len = ($len == 0x7e) ? 2 : ($len == 0x7f) ? 8 : 0;
 
-    my $len_len = ($len == 0x7e) ? 2 : ($len == 0x7f) ? 8 : 0;
-    my $len_buf = q<>;
-
-    my ($longs, $long);
+    my ($len_buf, $mask_buf);
 
     if ($len_len) {
-        $len_buf = $self->_read_with_buffer($len_len) or do {
+        $len_buf = $self->_read_with_buffer($len_len);
+
+        if (!$len_buf) {
             substr( $self->{'_partial_frame'}, 0, 0, $first2 );
-            return undef;
+            return defined($len_buf) ? q<> : undef;
         };
 
         if ($len_len == 2) {
             ($longs, $long) = ( 0, unpack('n', $len_buf) );
         }
         else {
+
+            #Do it this way to support 32-bit systems.
             ($longs, $long) = ( unpack('NN', $len_buf) );
         }
     }
     else {
         ($longs, $long) = ( 0, $len );
+        $len_buf = q<>;
     }
 
-    my $mask_buf;
     if ($mask_size) {
-        $mask_buf = $self->_read_with_buffer($mask_size) or do {
+        $mask_buf = $self->_read_with_buffer($mask_size);
+        if (!$mask_buf) {
             substr( $self->{'_partial_frame'}, 0, 0, $first2 . $len_buf );
-            return undef;
+            return defined($mask_buf) ? q<> : undef;
         };
     }
     else {
@@ -168,18 +180,20 @@ sub get_next_frame {
         #MacOS, at least, also chokes on sysread( 2**31, … )
         #(Is their size_t signed??), even on 64-bit.
         for ( 1 .. 4 ) {
-            $self->_append_chunk( 2**30, \$payload ) or do {
+            my $append_ok = $self->_append_chunk( 2**30, \$payload );
+            if (!$append_ok) {
                 substr( $self->{'_partial_frame'}, 0, 0, $first2 . $len_buf . $mask_buf . $payload );
-                return undef;
+                return defined($append_ok) ? q<> : undef;
             };
         }
     }
 
     if ($long) {
-        $self->_append_chunk( $long, \$payload ) or do {
+        my $append_ok = $self->_append_chunk( $long, \$payload );
+        if (!$append_ok) {
             substr( $self->{'_partial_frame'}, 0, 0, $first2 . $len_buf . $mask_buf . $payload );
-            return undef;
-        };
+            return defined($append_ok) ? q<> : undef;
+        }
     }
 
     $self->{'_partial_frame'} = q<>;
@@ -202,15 +216,20 @@ sub get_next_frame {
         $class;
     };
 
-    return $frame_class->create_from_parse(\$first2, \$len_len, \$mask_buf, \$payload);
+    return $frame_class->create_from_parse(\$first2, \$len_buf, \$mask_buf, \$payload);
 }
 
-#This will only return exactly the number of bytes requested.
-#If fewer than we want are available, then we return undef.
+# This will only return exactly the number of bytes requested.
+# If fewer than we want are available, then we return undef.
+# This incorporates the partial-frame buffer, which keeps get_next_frame()
+# a bit simpler than it otherwise might be.
+#
 sub _read_with_buffer {
     my ($self, $length) = @_;
 
-    #Prioritize the case where we read everything we need.
+    # Prioritize the case where we have everything we need.
+    # This will happen if, e.g., we got a partial frame on first read
+    # and a subsequent read has to pick back up.
 
     if ( length($self->{'_partial_frame'}) < $length ) {
         my $deficit = $length - length($self->{'_partial_frame'});
@@ -219,6 +238,9 @@ sub _read_with_buffer {
         if (!defined $read) {
             return undef;
         }
+        elsif (!length $read) {
+            return q<>;
+        }
 
         return substr($self->{'_partial_frame'}, 0, length($self->{'_partial_frame'}), q<>) . $read;
     }
@@ -239,6 +261,8 @@ sub _append_chunk {
         $cur_buf = $self->_read_with_buffer($length - $read_so_far);
         return undef if !defined $cur_buf;
 
+        return q<> if !length $cur_buf;
+
         $$buf_sr .= $cur_buf;
 
         last if (length($$buf_sr) - $start_buf_len) >= $length;
